package build

import (
	"sync"
)

// compile waits for the scan to finish and compiles all functions.
func compile(functions <-chan *Function, errors <-chan error) (Result, error) {
	result := Result{
		Functions: map[string]*Function{},
	}

	for functions != nil || errors != nil {
		select {
		case err, ok := <-errors:
			if !ok {
				errors = nil
				continue
			}

			return result, err

		case function, ok := <-functions:
			if !ok {
				functions = nil
				continue
			}

			result.Functions[function.Name] = function
		}
	}

	compileFunctions(result.Functions)

	for _, function := range result.Functions {
		if function.Error != nil {
			return result, function.Error
		}

		result.instructionCount += len(function.Assembler.Instructions)
	}

	return result, nil
}

// compileFunctions starts a goroutine for each function compilation and waits for completion.
func compileFunctions(functions map[string]*Function) {
	wg := sync.WaitGroup{}

	for _, function := range functions {
		wg.Add(1)

		go func() {
			defer wg.Done()
			function.Compile()
		}()
	}

	wg.Wait()
}
